import React from 'react'
import { connect } from './init'
import collectionActions from './collection-actions'
import { optionRequireRefetch } from './helper'

export default rootOptions => WrappedComponent =>  {

    const Container = class extends React.Component {
      componentDidMount() {
        if(rootOptions.awake) {
          this.props.fetchIfNeeded({pageIndex:0})
        }
      }
      componentDidUpdate(prevProps) {
        if(optionRequireRefetch(prevProps.recentOption, this.props.recentOption)) {
            this.props.fetchIfNeeded({...this.props.recentOption, pageIndex:0})
          }
      }
      componentWillUnmount() {
        this.props.unsubscribeIfNeeded()
      }
      render() {
        return <WrappedComponent
                  // pass through the connected redux props
                  {...this.props}

                  // api to query the first page
                  query={(options)=>{
                    options = options || {}
                    this.props.fetchIfNeeded({
                      ...options,
                      pageIndex: options.pageIndex || 0
                    })
                  }}

                  // api to query next pages
                  queryNext={()=>{
                    this.props.fetchIfNeeded({
                      ...this.props.recentOption,
                      pageIndex: this.props.lastSuccessPageIndex + 1
                    })
                  }}/>
      }
    }

    return connect()(
      (store, {options}) => {
        var state = store.getIn(['firestore','getFsState'])({...rootOptions,...options})
        if(!state)
          return {}
        // collect user objects from pages
        var u1 = state.pages && Object.values(state.pages).reduce((obj, page)=>Object.assign({}, obj, page.items), {})
        // convert the object to ui-ready data array
        var u2 = u1 && Object.entries(u1).map(entry=>{return {id:entry[0], ...entry[1]}})
        return {
          items: u2,
          pages: state.pages,
          recentOption: state.lastOption,
          lastSuccessPageIndex: state.lastSuccessOption ? state.lastSuccessOption.pageIndex : -1,
          fetching: state.pages && !!Object.values(state.pages).find(page=>page.isFetching),
          hasMore: !state.ended && (!state.lastOption || state.lastOption.pageLength),
        }
      },
      (dispatch, {options}) => {
        const actions = collectionActions()
        const componentOptions = {...rootOptions, ...options}
        return {
          fetchIfNeeded: (functionOptions) => {
            var params = {...componentOptions, ...functionOptions}
            if(params.sync)
              params.onSnapshot = result => dispatch(actions.onSnapshot(params, result))
            dispatch(actions.fetchIfNeeded(params))
          },
          unsubscribeIfNeeded: (functionOptions) => {
            var params = {...componentOptions, ...functionOptions}
            dispatch(actions.unsubscribeIfNeeded(params))
          }
        }
      }
    )(Container)
}
