import { optionRequireRefetch } from './helper'
import {
  REQUEST,
  RESPONSE_SUCCESS,
  RESPONSE_FAIL,
  SUBSCRIBE,
  UNSUBSCRIBE
} from './collection-actions'

function page(state = {
    isFetching : false,
    fetched : false,
    items : {},
    itemIds : [],
    error : null
}, action) {
  switch(action.type) {
    case REQUEST(action.options):
      return Object.assign({}, state, {
        isFetching: true,
        fetched: false,
        error: null
      })
    case RESPONSE_SUCCESS(action.options):
      return Object.assign({}, state, {
        isFetching: false,
        fetched: true,
        items : action.result,
        itemIds: Object.keys(action.result),
        error: null
      })
    case RESPONSE_FAIL(action.options):
      return Object.assign({}, state, {
        isFetching: false,
        fetched: false,
        items: {},
        itemIds: [],
        error: action.error
      })
    default:
      return state
  }
}

export default (state, action) => {
  switch(action.type) {
    case REQUEST(action.options):
    case RESPONSE_SUCCESS(action.options):
    case RESPONSE_FAIL(action.options):
      var _pages = state.pages
      var _options = state.lastOption
      var _lastSuccessOptions = state.lastSuccessOption

      // TODO: check where change
      //var _nameTagChanged = _options && _options.nametag !== null && _options.nametag !== action.options.nametag
      //var _pageLengthChanged = _options && _options.pageLength !== null && _options.pageLength !== action.options.pageLength
      //var optionPatternChanged = _nameTagChanged || _pageLengthChanged

      const invalidate = optionRequireRefetch(_options,action. options)
      if(invalidate) {
        action.options.pageIndex = 0
        _pages = {}
        _lastSuccessOptions = null
      }

      var newPage = page(_pages && _pages[action.options.pageIndex], action)

      return Object.assign({}, state, {
        lastOption : action.options,
        pages: Object.assign({}, _pages, {
          [action.options.pageIndex]: newPage
        }),
        lastSuccessOption: action.type === RESPONSE_SUCCESS(action.options)
          && newPage.fetched && newPage.itemIds.length > 0
          ? action.options : _lastSuccessOptions,
        ended: !invalidate && action.type === RESPONSE_SUCCESS(action.options)
          && newPage.fetched && newPage.itemIds.length === 0
      })

    case SUBSCRIBE(action.options):
      return Object.assign({}, state, {
        unsubscribe: action.unsubscribe
      })
    case UNSUBSCRIBE(action.options):
      if(state.unsubscribe) {
        state.unsubscribe()
      }
      return Object.assign({}, state, {
        unsubscribe: null
      })
    default:
      return state
  }
}
