import namer from './namer'
import { RESPONSE_SUCCESS as COLLECTION_RESPONSE_SUCCESS } from './collection-actions'
import {
  REQUEST as DOCUMENT_REQUEST,
  RESPONSE_SUCCESS as DOCUMENT_RESPONSE_SUCCESS,
  RESPONSE_FAIL as DOCUMENT_RESPONSE_FAIL,
  SUBSCRIBE as DOCUMENT_SUBSCRIBE,
  UNSUBSCRIBE as DOCUMENT_UNSUBSCRIBE
} from './document-actions'

import documentReducer from './document-reducer'
import collectionReducer from './collection-reducer'
import { fromJS, Map } from 'immutable'

export default () => {
  const databaseReducer = (state, action) => {
    // cross-reducer, paginated query -> global
    switch(action.type) {
      case COLLECTION_RESPONSE_SUCCESS(action.options) :
        return Object.assign({}, state, {
          items:Object.assign({}, state.items, Object.entries(action.result).reduce((obj,e)=>{
            obj[e[0]] = {
              data:e[1],
              isFetching: false,
              didInvalidate: false,
              error: null
            }
            return obj
          },{}))
        })
      case DOCUMENT_REQUEST(action.options):
      case DOCUMENT_RESPONSE_SUCCESS(action.options):
      case DOCUMENT_RESPONSE_FAIL(action.options):
      case DOCUMENT_SUBSCRIBE(action.options):
      case DOCUMENT_UNSUBSCRIBE(action.options):
        return Object.assign({}, state, {
          items: Object.assign({}, state.items, {
            [action.options.item_id] : documentReducer(state.items&&state.items[action.options.item_id]||{}, action)
          })
        })
      default :
        return state
    }
  }

  const isCollectionMode = (action) => action.options.mode==='search' || action.options.mode==='query'

  const getReducer = (state, action) => {
    if(isCollectionMode(action))
      return collectionReducer(state, action)
    else
      return databaseReducer(state, action)
  }

  const autoInit = (rootFsState, action) => {
    var name = namer(action.options)
    if(name && !rootFsState[name]) {
      // auto create state, modify origin
      rootFsState = {...rootFsState}
      rootFsState[name] = {}

      if(isCollectionMode(action)) {
        rootFsState[namer({...action.options,mode:'database'})] = {}
      }
    }
    return rootFsState
  }

  const reduce = (rootFsState, action) => {
    var newRootState = {...rootFsState}
    var name = namer(action.options)
    if(name && newRootState[name]) {
      newRootState[name] = getReducer(newRootState[name], action)
      if(isCollectionMode(action)) {
        const dbStateName = namer({...action.options,mode:'database'})
        newRootState[dbStateName] = databaseReducer(newRootState[dbStateName], action)
      }
    }
    return newRootState
  }

  return (state = Map(), action) => {
    var newState = state.toJS()
    newState = autoInit(newState, action)
    var reduced = reduce(newState, action)
    reduced.getFsState = options => reduced[namer(options)]
    return fromJS(reduced)
  }
}
