import { document } from '@metromerce/firestore'

import namer from './namer'
export const REQUEST = options => namer(options) + '_DOCUMENT_REQUEST'
export const RESPONSE_SUCCESS = options => namer(options) + '_DOCUMENT_RESPONSE_SUCCESS'
export const RESPONSE_FAIL = options => namer(options) + '_DOCUMENT_RESPONSE_FAIL'
export const SUBSCRIBE = options => namer(options) + '_DOCUMENT_SUBSCRIBE'
export const UNSUBSCRIBE = options => namer(options) + '_DOCUMENT_UNSUBSCRIBE'

function fetchIfNeeded(options) {
  return (dispatch, getState) => {
    dispatch(unsubscribeIfNeeded(options))
    const db = getState().getIn(['firestore','getFsState'])(options)
    if(shouldFetch(db, options))
      return dispatch(fetch(options))
    else {
      return Promise.resolve()
    }
  }
}

function shouldFetch(db, options) {
  if(options.onSnapshot)
    return true // always fetch if subscribing regardless of the cache exist
  if(db && !db.items[options.item_id])
    return true
  if(db && db.items[options.item_id].isFetching)
    return false
  return !db || db.items[options.item_id].didInvalidate
}

function fetch(options) {
  return dispatch => {
    dispatch(request(options))
    var queryParams = {...options}
    if(queryParams.onSnapshot) {
      dispatch(unsubscribeIfNeeded(options))
      const unsubscribe = document.onSnapshot(queryParams)
      dispatch(subscribe(options, unsubscribe))
    }
    else {
      document.get(queryParams)
        .then(item => {
          dispatch(responseSucesss(options, item))
        })
        .catch(error => {
          dispatch(responseFail(options, error))
        })
    }
  }
}

function request(options) {
  return { type:REQUEST(options), options }
}

function responseSucesss(options, result) {
  return dispatch => {
    dispatch({ type:RESPONSE_SUCCESS(options), options, result })
    if(options.callback)
      options.callback(result)
  }
}

function responseFail(options, error) {
  return { type:RESPONSE_FAIL(options), options, error }
}

function subscribe(options, unsubscribe ) {
  return { type:SUBSCRIBE(options), options, unsubscribe }
}
function unsubscribeIfNeeded(options) {
  return (dispatch, getState) => {
    //const state = getState().firestore
    const db = getState().getIn(['firestore','getFsState'])(options)
    if(db && db.items[options.item_id] && db.items[options.item_id].unsubscribe)
      dispatch({ type:UNSUBSCRIBE(options), options })
    else
      Promise.resolve()
  }
}

export default () => {
  return {
    fetchIfNeeded: (options) => {
      return fetchIfNeeded(options)
    },
    onSnapshot: (options, result) => {
      return responseSucesss(options, result)
    },
    unsubscribeIfNeeded: (options) => {
      return unsubscribeIfNeeded(options)
    }
  }
}
