import React from 'react'
import { connect } from './init'
import documentActions from './document-actions'
import namer from './namer'
import { optionRequireRefetch } from './helper'

export default rootOptions => WrappedComponent =>  {

    const Container = class extends React.Component {
      componentDidMount() {
        this.props.fetchIfNeeded()
      }
      componentDidUpdate(prevProps) {
        if(optionRequireRefetch(prevProps.recentOption, this.props.recentOption)) {
          this.props.fetchIfNeeded()
        }
      }
      componentWillUnmount() {
        this.props.unsubscribeIfNeeded()
      }
      render() {
        return <WrappedComponent
                  // pass through the connected redux props
                  {...this.props}
                />
      }
    }

    return connect()(
      (store, {options}) => {
        const componentOptions = {...rootOptions, ...options}
        var state = store.getIn(['firestore','getFsState'])(componentOptions)
        if(!state) {
          return { item : {} }
        }
        return {
          item : {
            id: componentOptions.item_id,
            data: state.items[componentOptions.item_id] ? state.items[componentOptions.item_id].data : null,
            state: state.items[componentOptions.item_id]
          },
          recentOption: componentOptions
        }
      },
      (dispatch, {options}) => {
        const actions = documentActions()
        return {
          fetchIfNeeded: (functionOptions) => {
            var params = {...rootOptions, ...options, ...functionOptions}
            if(params.sync)
              params.onSnapshot = result => dispatch(actions.onSnapshot(params, result))
            dispatch(actions.fetchIfNeeded(params))
          },
          unsubscribeIfNeeded: (functionOptions) => {
            var params = {...rootOptions, ...options, ...functionOptions}
            dispatch(actions.unsubscribeIfNeeded(params))
          }
        }
      }
    )(Container)
}
